#!/usr/bin/perl

# m3ufs.pl - Fuse based virtual filesystem which allows a live view into the playlists stored on the sqeezebox server without copying or moving files (uses symlinks). 
# Copyright (C) 2009 Marco Schinkel (schinkelm at web.de)

# This program is free software; you can redistribute it and/or modify it under the terms of the GNU General Public License as
# published by the Free Software Foundation; either version 3 of the License, or (at your option) any later version.
# This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
# You should have received a copy of the GNU General Public License along with this program; if not, see <http://www.gnu.org/licenses/>.

use strict;
use warnings;

use Encode;
use Data::Dumper;
use FindBin::Real;
use MP3::Info qw(:all);
use File::Glob qw(:globally :nocase);
use File::Spec::Functions qw(abs2rel); 
use File::Basename;
use Fuse::Simple qw(:usual fs_flush);

use_mp3_utf8(0);
my $playlist_dir = FindBin::Real::Bin();

my %playlist_cache = ();
my %playlist_mtimes = ();

my @reserved_characters = (
  '|', '\\', '?', '*', '<', '"', ':', '>', '+', '[', ']', '/'
);


my $mountpoint = $ARGV[0];

sub char_replace {
  my ($string, @chars) = @_;
  foreach my $c (@chars) {
    $string =~ s/\Q$c\E/ /g;
  }
  
  return $string;
}


# return a list of existing media files in a playlist
sub playlist_parse {
  my ($name, $suffix) = @_;

  my $path = $playlist_dir . '/' . $name . $suffix;  
  return () unless -e $path;
  
  my $m3u;
  open($m3u, '<:utf8', $path) or return ();  
  
  $playlist_mtimes{$name} = (stat $path)[9];  
  return grep { -e $_ } grep { $_ !~ /^#/ } map { chomp; $_ } <$m3u>;  
}


# return files in a playlist
sub playlist_get_entries {
  my ($name, $dir, $suffix) = @_;
  my $path = $dir . '/' . $name . $suffix;  

  return $playlist_cache{$name}
    if (exists $playlist_cache{$name} && $playlist_mtimes{$name} < (stat $path)[9]);

  my %files = ();
  my $playlist_pos = 1;
  foreach my $mp3_path (playlist_parse($name, $suffix)) {      
    my ($name, $dir, $suffix) = fileparse($mp3_path, qr/.mp3$/i);
    
    my $playlist_pos_name = sprintf('%03d', $playlist_pos);  
    if (my $mp3_info = get_mp3tag($mp3_path)) {
      $playlist_pos_name .= ' ' . $mp3_info->{ARTIST} . ' - ' . $mp3_info->{TITLE} . $suffix;
      
      $playlist_pos_name = Encode::decode('iso-8859-1', char_replace($playlist_pos_name, @reserved_characters));
      utf8::downgrade($playlist_pos_name);
    } else {
      $playlist_pos_name .= ' ' . char_replace($name, @reserved_characters) . $suffix;
    }
    
    $mp3_path = '../' . abs2rel($mp3_path, $mountpoint);
              
    $files{$playlist_pos_name} = \$mp3_path;      
    $playlist_pos++;
  }

  $playlist_cache{$name} = \%files;
  return \%files;
}

# filesystem root
sub m3ufs() {
  my %root;
  foreach my $playlist_path (glob "$playlist_dir/*.m3u") {      
    my @pl = fileparse($playlist_path, qr/.m3u$/i);
    $root{$pl[0]} = sub { playlist_get_entries(@pl); };
  }

  return \%root;  
}

my $flush = 60;
local $SIG{ALRM} = sub { fs_flush('/'); alarm $flush; };
alarm $flush;

main(
  'mountpoint'	=> $mountpoint,		# actually optional
  'mountopts'	=> 'allow_other',
  
  'debug'	=> 0,           	# for debugging Fuse::Simple. optional
  'fuse_debug'	=> 0,           	# for debugging FUSE itself. optional
  'threaded'	=> 0,           	# optional
  '/'		=> \&m3ufs,		# required :-)
);
                                                                          